@@ -3,11 +3,13 @@
 
 /* ANTLR Translator Generator
  * Project led by Terence Parr at http://www.jGuru.com
- * Software rights: http://www.antlr.org/RIGHTS.html
+ * Software rights: http://www.antlr.org/license.html
  *
- * $Id: CharBuffer.hpp,v 1.1.1.1 2003/04/06 22:26:24 braden Exp $
+ * $Id: CharBuffer.hpp,v 1.1.1.2 2004/11/08 20:45:24 braden Exp $
  */
 
+#include <istream>
+
 #include <antlr/config.hpp>
 #include <antlr/InputBuffer.hpp>
 
