@@ -3,9 +3,9 @@
 
 /* ANTLR Translator Generator
  * Project led by Terence Parr at http://www.jGuru.com
- * Software rights: http://www.antlr.org/RIGHTS.html
+ * Software rights: http://www.antlr.org/license.html
  *
- * $Id: CommonAST.hpp,v 1.1.1.1 2003/04/06 22:26:27 braden Exp $
+ * $Id: CommonAST.hpp,v 1.1.1.2 2004/11/08 20:45:24 braden Exp $
  */
 
 #include <antlr/config.hpp>
@@ -42,6 +42,7 @@ public:
 
 	static RefAST factory();
 
+	static const char* const TYPE_NAME;
 protected:
 	int ttype;
 	ANTLR_USE_NAMESPACE(std)string text;
