@@ -3,13 +3,14 @@
 
 /* ANTLR Translator Generator
  * Project led by Terence Parr at http://www.jGuru.com
- * Software rights: http://www.antlr.org/RIGHTS.html
+ * Software rights: http://www.antlr.org/license.html
  *
- * $Id: IOException.hpp,v 1.1.1.1 2003/04/06 22:26:27 braden Exp $
+ * $Id: IOException.hpp,v 1.1.1.2 2004/11/08 20:45:24 braden Exp $
  */
 
 #include <antlr/config.hpp>
 #include <antlr/ANTLRException.hpp>
+#include <exception>
 
 #ifdef ANTLR_CXX_SUPPORTS_NAMESPACE
 namespace antlr {
