@@ -3,9 +3,9 @@
 
 /* ANTLR Translator Generator
  * Project led by Terence Parr at http://www.jGuru.com
- * Software rights: http://www.antlr.org/RIGHTS.html
+ * Software rights: http://www.antlr.org/license.html
  *
- * $Id: String.hpp,v 1.1.1.1 2003/04/06 22:26:27 braden Exp $
+ * $Id: String.hpp,v 1.1.1.2 2004/11/08 20:45:24 braden Exp $
  */
 
 #include <antlr/config.hpp>
@@ -15,9 +15,9 @@
 namespace antlr {
 #endif
 
-ANTLR_USE_NAMESPACE(std)string operator+(const ANTLR_USE_NAMESPACE(std)string& lhs,const int rhs);
+ANTLR_API ANTLR_USE_NAMESPACE(std)string operator+(const ANTLR_USE_NAMESPACE(std)string& lhs,const int rhs);
 
-ANTLR_USE_NAMESPACE(std)string charName(int ch);
+ANTLR_API ANTLR_USE_NAMESPACE(std)string charName(int ch);
 
 #ifdef ANTLR_CXX_SUPPORTS_NAMESPACE
 }
