@@ -2,8 +2,8 @@
 //
 // OpenVRML
 //
-// Copyright (C) 2001  S. K. Bose
-// Copyright (C) 2003  Braden McDaniel
+// Copyright 2001  S. K. Bose
+// Copyright 2003, 2004  Braden McDaniel
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -21,6 +21,7 @@
 //
 
 # include <numeric>
+# include <ostream>
 # include "private.h"
 # include "basetypes.h"
 
@@ -58,28 +59,42 @@ namespace openvrml {
  */
 
 /**
- * @fn color::color() throw ()
- *
  * @brief Construct.
  */
+color::color() throw ()
+{}
 
 /**
- * @fn color::color(float (&rgb)[3]) throw ()
- *
  * @brief Construct from a 3-element float array.
  *
  * @param rgb   an array comprising red, green, and blue components.
  */
+color::color(const float (&rgb)[3]) throw ()
+{
+    assert(rgb[0] >= 0.0 && rgb[0] <= 1.0);
+    this->rgb[0] = rgb[0];
+    assert(rgb[1] >= 0.0 && rgb[1] <= 1.0);
+    this->rgb[1] = rgb[1];
+    assert(rgb[2] >= 0.0 && rgb[2] <= 1.0);
+    this->rgb[2] = rgb[2];
+}
 
 /**
- * @fn color::color(float r, float g, float b) throw ()
- *
  * @brief Construct from red, green, and blue components.
  *
  * @param r red component.
  * @param g green component.
  * @param b blue component.
  */
+color::color(float r, float g, float b) throw ()
+{
+    assert(r >= 0.0 && r <= 1.0);
+    this->rgb[0] = r;
+    assert(g >= 0.0 && g <= 1.0);
+    this->rgb[1] = g;
+    assert(b >= 0.0 && b <= 1.0);
+    this->rgb[2] = b;
+}
 
 /**
  * @fn const float & color::operator[](const size_t index) const throw ()
@@ -207,7 +222,7 @@ void color::hsv(float h, const float s, const float v) throw ()
 }
 
 /**
- * @relates openvrml::color
+ * @relatesalso color
  *
  * @brief Compare for equality.
  *
@@ -224,9 +239,7 @@ bool operator==(const color & lhs, const color & rhs) throw ()
 }
 
 /**
- * @fn bool operator!=(const color & lhs, const color & rhs) throw ()
- *
- * @relates openvrml::color
+ * @relatesalso color
  *
  * @brief Compare for inequality.
  *
@@ -236,9 +249,13 @@ bool operator==(const color & lhs, const color & rhs) throw ()
  * @return @c true if @p lhs and @p rhs dot not have the same value; @c false
  *      otherwise.
  */
+bool operator!=(const color & lhs, const color & rhs) throw ()
+{
+    return !(lhs == rhs);
+}
 
 /**
- * @relates color
+ * @relatesalso color
  *
  * @brief Stream output.
  *
@@ -270,42 +287,69 @@ std::ostream & operator<<(std::ostream & out, const color & c)
  */
 
 /**
- * @fn vec2f::vec2f() throw ()
- *
  * @brief Construct.
  */
+vec2f::vec2f() throw ()
+{
+    this->vec[0] = 0.0;
+    this->vec[1] = 0.0;
+}
 
 /**
- * @fn vec2f::vec2f(const float (&vec)[2]) throw ()
- *
  * @brief Construct from an array.
  *
+ * @pre Elements of @p vec are valid numeric values (i.e., not NaN).
+ *
  * @param vec   an array comprising the vector components.
  */
+vec2f::vec2f(const float (&vec)[2]) throw ()
+{
+    //
+    // Ensure elements of vec are not NaN.
+    //
+    assert(vec[0] == vec[0]);
+    assert(vec[1] == vec[1]);
+
+    this->vec[0] = vec[0];
+    this->vec[1] = vec[1];
+}
 
 /**
- * @fn vec2f::vec2f(float x, float y) throw ()
- *
  * @brief Construct from @p x and @p y components.
  *
+ * @pre @p x and @p y are valid numeric values (i.e., not NaN).
+ *
  * @param x x component.
  * @param y y component.
  */
+vec2f::vec2f(const float x, const float y) throw ()
+{
+    //
+    // Ensure x and y are not NaN.
+    //
+    assert(x == x);
+    assert(y == y);
+
+    this->vec[0] = x;
+    this->vec[1] = y;
+}
 
 /**
- * @fn vec2f & vec2f::operator*=(float scalar) throw ()
- *
  * @brief Multiply by a scalar.
  *
  * @param scalar    factor by which to multiply.
  *
  * @return a reference to the object.
  */
+vec2f & vec2f::operator*=(const float scalar) throw ()
+{
+    this->vec[0] *= scalar;
+    this->vec[1] *= scalar;
+    return *this;
+}
 
 /**
- * @fn const vec2f operator*(const vec2f & lhs, float rhs) throw ()
- *
- * @relates openvrml::vec2f
+ * @relatesalso vec2f
  *
  * @brief Multiply a vector by a scalar.
  *
@@ -314,44 +358,79 @@ std::ostream & operator<<(std::ostream & out, const color & c)
  *
  * @return the result vector.
  */
+const vec2f operator*(const vec2f & lhs, const float rhs) throw ()
+{
+    vec2f result(lhs);
+    return result *= rhs;
+}
 
 /**
- * @fn vec2f & vec2f::operator/=(float scalar) throw ()
+ * @relatesalso vec2f
  *
+ * @brief Multiply a vector by a scalar.
+ *
+ * @param lhs   left-hand operand (the scalar).
+ * @param rhs   right-hand operand (the vector).
+ *
+ * @return the result vector.
+ */
+const vec2f operator*(const float lhs, const vec2f & rhs) throw ()
+{
+    vec2f result(rhs);
+    return result *= lhs;
+}
+
+/**
  * @brief Divide by a scalar.
  *
- * @param scalar    factor by which to multiply.
+ * @pre @p scalar is nonzero.
+ *
+ * @param scalar    divisor.
  *
  * @return a reference to the object.
  */
+vec2f & vec2f::operator/=(const float scalar) throw ()
+{
+    assert(scalar != 0.0);
+    this->vec[0] /= scalar;
+    this->vec[1] /= scalar;
+    return *this;
+}
 
 /**
- * @fn const vec2f operator/(const vec2f & lhs, float rhs) throw ()
- *
- * @relates openvrml::vec2f
+ * @relatesalso vec2f
  *
  * @brief Divide a vector by a scalar.
  *
+ * @pre @p rhs is nonzero.
+ *
  * @param lhs   left-hand operand (the vector).
  * @param rhs   right-hand operand (the scalar).
  *
  * @return the result vector.
  */
+const vec2f operator/(const vec2f & lhs, const float rhs) throw ()
+{
+    vec2f result(lhs);
+    return result /= rhs;
+}
 
 /**
- * @fn vec2f & vec2f::operator+=(const vec2f & vec) throw ()
- *
  * @brief Add a vector.
  *
  * @param vec   the vector to add.
  *
  * @return a reference to the object.
  */
+vec2f & vec2f::operator+=(const vec2f & vec) throw ()
+{
+    this->vec[0] += vec[0];
+    this->vec[1] += vec[1];
+    return *this;
+}
 
 /**
- * @fn const vec2f operator+(const vec2f & lhs, const vec2f & rhs) throw ()
- *
- * @relates openvrml::vec2f
+ * @relatesalso vec2f
  *
  * @brief Add two vectors.
  *
@@ -360,21 +439,28 @@ std::ostream & operator<<(std::ostream & out, const color & c)
  *
  * @return the result vector.
  */
+const vec2f operator+(const vec2f & lhs, const vec2f & rhs) throw ()
+{
+    vec2f result(lhs);
+    return result += rhs;
+}
 
 /**
- * @fn vec2f & vec2f::operator-=(const vec2f & vec) throw ()
- *
  * @brief Subtract a vector.
  *
  * @param vec   the vector to subtract.
  *
  * @return a reference to the object.
  */
+vec2f & vec2f::operator-=(const vec2f & vec) throw ()
+{
+    this->vec[0] -= vec[0];
+    this->vec[1] -= vec[1];
+    return *this;
+}
 
 /**
- * @fn const vec2f operator-(const vec2f & lhs, const vec2f & rhs) throw ()
- *
- * @relates openvrml::vec2f
+ * @relatesalso vec2f
  *
  * @brief Subtract two vectors.
  *
@@ -383,14 +469,21 @@ std::ostream & operator<<(std::ostream & out, const color & c)
  *
  * @return the result vector.
  */
+const vec2f operator-(const vec2f & lhs, const vec2f & rhs) throw ()
+{
+    vec2f result(lhs);
+    return result -= rhs;
+}
 
 /**
- * @fn const vec2f vec2f::operator-() const throw ()
- *
  * @brief Negate.
  *
  * @return the additive inverse of the vector.
  */
+const vec2f vec2f::operator-() const throw ()
+{
+    return vec2f(-this->vec[0], -this->vec[1]);
+}
 
 /**
  * @fn const float & vec2f::operator[](const size_t index) const throw ()
@@ -405,19 +498,6 @@ std::ostream & operator<<(std::ostream & out, const color & c)
  * @pre @p index is less than 2.
  */
 
-/**
- * @fn float & vec2f::operator[](const size_t index) throw ()
- *
- * @brief Index-based component access.
- *
- * @param index 0 corresponds to the x component; 1 corresponds to the y
- *              component.
- *
- * @return a reference to the component corresponding to @p index.
- *
- * @pre @p index is less than 2.
- */
-
 /**
  * @fn float vec2f::x() const throw ()
  *
@@ -439,6 +519,8 @@ std::ostream & operator<<(std::ostream & out, const color & c)
  *
  * @brief Set the x component.
  *
+ * @pre @p value is a valid numeric value (i.e., not NaN).
+ *
  * @param value new x component value.
  */
 
@@ -447,26 +529,33 @@ std::ostream & operator<<(std::ostream & out, const color & c)
  *
  * @brief Set the y component.
  *
+ * @pre @p value is a valid numeric value (i.e., not NaN).
+ *
  * @param value new y component value.
  */
 
 /**
- * @fn float vec2f::dot(const vec2f & vec) const throw ()
- *
  * @brief Dot product.
  *
  * @param vec
  *
  * @return the dot product of the vector and @p vec.
  */
+float vec2f::dot(const vec2f & vec) const throw ()
+{
+    return this->vec[0] * vec[0]
+        + this->vec[1] * vec[1];
+}
 
 /**
- * @fn float vec2f::length() const throw ()
- *
  * @brief Geometric length.
  *
  * @return the length of the vector.
  */
+float vec2f::length() const throw ()
+{
+    return float(sqrt(this->dot(*this)));
+}
 
 /**
  * @brief Normalize.
@@ -483,7 +572,7 @@ const vec2f vec2f::normalize() const throw ()
 }
 
 /**
- * @relates vec2f
+ * @relatesalso vec2f
  *
  * @brief Compare for equality.
  *
@@ -500,9 +589,7 @@ bool operator==(const vec2f & lhs, const vec2f & rhs) throw ()
 }
 
 /**
- * @fn bool operator!=(const vec2f & lhs, const vec2f & rhs) throw ()
- *
- * @relates openvrml::vec2f
+ * @relatesalso vec2f
  *
  * @brief Compare for inequality.
  *
@@ -512,9 +599,13 @@ bool operator==(const vec2f & lhs, const vec2f & rhs) throw ()
  * @return @c true if @p lhs and @p rhs dot not have the same value; @c false
  *         otherwise.
  */
+bool operator!=(const vec2f & lhs, const vec2f & rhs) throw ()
+{
+    return !(lhs == rhs);
+}
 
 /**
- * @relates vec2f
+ * @relatesalso vec2f
  *
  * @brief Stream output.
  *
@@ -546,43 +637,78 @@ std::ostream & operator<<(std::ostream & out, const vec2f & v)
  */
 
 /**
- * @fn vec3f::vec3f() throw ()
- *
  * @brief Construct.
  */
+vec3f::vec3f() throw ()
+{
+    this->vec[0] = 0.0;
+    this->vec[1] = 0.0;
+    this->vec[2] = 0.0;
+}
 
 /**
- * @fn vec3f::vec3f(const float (&vec)[3]) throw ()
- *
  * @brief Construct from an array.
  *
+ * @pre Elements of @p vec are valid numeric values (i.e., not NaN).
+ *
  * @param vec   an array comprising the vector components.
  */
+vec3f::vec3f(const float (&vec)[3]) throw ()
+{
+    //
+    // Ensure the elements of vec are not NaN.
+    //
+    assert(vec[0] == vec[0]);
+    assert(vec[1] == vec[1]);
+    assert(vec[2] == vec[2]);
+
+    this->vec[0] = vec[0];
+    this->vec[1] = vec[1];
+    this->vec[2] = vec[2];
+}
 
 /**
- * @fn vec3f::vec3f(float x, float y, float z) throw ()
- *
  * @brief Construct from @p x, @p y, and @p z components.
  *
+ * @pre @p x, @p y, and @p z are valid numeric values (i.e., not NaN).
+ *
  * @param x x component.
  * @param y y component.
  * @param z z component.
  */
+vec3f::vec3f(const float x, const float y, const float z) throw ()
+{
+    //
+    // Ensure x, y, and z are not NaN.
+    //
+    assert(x == x);
+    assert(y == y);
+    assert(z == z);
+
+    this->vec[0] = x;
+    this->vec[1] = y;
+    this->vec[2] = z;
+}
 
 /**
- * @fn vec3f & vec3f::operator*=(const vec3f & vec) throw ()
- *
  * @brief Cross multiply.
  *
  * @param vec   vector by which to multiply.
  *
  * @return a reference to the object.
  */
+vec3f & vec3f::operator*=(const vec3f & vec) throw ()
+{
+    vec3f temp;
+    temp.x(this->y() * vec.z() - this->z() * vec.y());
+    temp.y(this->z() * vec.x() - this->x() * vec.z());
+    temp.z(this->x() * vec.y() - this->y() * vec.x());
+    *this = temp;
+    return *this;
+}
 
 /**
- * @fn const vec3f operator*(const vec3f & lhs, const vec3f & rhs) throw ()
- *
- * @relates openvrml::vec3f
+ * @relatesalso vec3f
  *
  * @brief Cross multiply two vectors.
  *
@@ -591,6 +717,11 @@ std::ostream & operator<<(std::ostream & out, const vec2f & v)
  *
  * @return the result vector.
  */
+const vec3f operator*(const vec3f & lhs, const vec3f & rhs) throw ()
+{
+    vec3f result(lhs);
+    return result *= rhs;
+}
 
 /**
  * @brief Multiply by a matrix.
@@ -616,20 +747,23 @@ vec3f & vec3f::operator*=(const mat4f & mat) throw ()
 }
 
 /**
- * @fn const vec3f operator*(const vec3f & vec, const mat4f & mat) throw ()
- *
- * @relates openvrml::vec3f
+ * @relatesalso vec3f
  *
  * @brief Multiply a vector by a matrix.
  *
- * @param mat   a matrix.
  * @param vec   a 3-component vector.
+ * @param mat   a matrix.
  *
  * @return the result vector.
  */
+const vec3f operator*(const vec3f & vec, const mat4f & mat) throw ()
+{
+    vec3f result(vec);
+    return result *= mat;
+}
 
 /**
- * @relates vec3f
+ * @relatesalso vec3f
  *
  * @brief Multiply a matrix by a vector.
  *
@@ -652,19 +786,22 @@ const vec3f operator*(const mat4f & mat, const vec3f & vec) throw ()
 }
 
 /**
- * @fn vec3f & vec3f::operator*=(float scalar) throw ()
- *
  * @brief Multiply by a scalar.
  *
  * @param scalar    factor by which to multiply.
  *
  * @return a reference to the object.
  */
+vec3f & vec3f::operator*=(const float scalar) throw ()
+{
+    this->vec[0] *= scalar;
+    this->vec[1] *= scalar;
+    this->vec[2] *= scalar;
+    return *this;
+}
 
 /**
- * @fn const vec3f operator*(const vec2f & lhs, float rhs) throw ()
- *
- * @relates openvrml::vec3f
+ * @relatesalso vec3f
  *
  * @brief Multiply a vector by a scalar.
  *
@@ -673,44 +810,81 @@ const vec3f operator*(const mat4f & mat, const vec3f & vec) throw ()
  *
  * @return the result vector.
  */
+const vec3f operator*(const vec3f & lhs, const float rhs) throw ()
+{
+    vec3f result(lhs);
+    return result *= rhs;
+}
 
 /**
- * @fn vec3f & vec3f::operator/=(float scalar) throw ()
+ * @relatesalso vec3f
  *
+ * @brief Multiply a vector by a scalar.
+ *
+ * @param lhs   left-hand operand (the scalar).
+ * @param rhs   right-hand operand (the vector).
+ *
+ * @return the result vector.
+ */
+const vec3f operator*(const float lhs, const vec3f & rhs) throw ()
+{
+    vec3f result(rhs);
+    return result *= lhs;
+}
+
+/**
  * @brief Divide by a scalar.
  *
- * @param scalar    factor by which to multiply.
+ * @pre @p scalar is nonzero.
+ *
+ * @param scalar    divisor.
  *
  * @return a reference to the object.
  */
+vec3f & vec3f::operator/=(const float scalar) throw ()
+{
+    assert(scalar != 0.0);
+    this->vec[0] /= scalar;
+    this->vec[1] /= scalar;
+    this->vec[2] /= scalar;
+    return *this;
+}
 
 /**
- * @fn const vec3f operator/(const vec3f & lhs, float rhs) throw ()
- *
- * @relates openvrml::vec3f
+ * @relatesalso vec3f
  *
  * @brief Divide a vector by a scalar.
  *
+ * @pre @p rhs is nonzero.
+ *
  * @param lhs   left-hand operand (the vector).
  * @param rhs   right-hand operand (the scalar).
  *
  * @return the result vector.
  */
+const vec3f operator/(const vec3f & lhs, const float rhs) throw ()
+{
+    vec3f result(lhs);
+    return result /= rhs;
+}
 
 /**
- * @fn vec3f & vec3f::operator+=(const vec3f & vec) throw ()
- *
  * @brief Add a vector.
  *
  * @param vec   the vector to add.
  *
  * @return a reference to the object.
  */
+vec3f & vec3f::operator+=(const vec3f & vec) throw ()
+{
+    this->vec[0] += vec[0];
+    this->vec[1] += vec[1];
+    this->vec[2] += vec[2];
+    return *this;
+}
 
 /**
- * @fn const vec3f operator+(const vec3f & lhs, const vec3f & rhs) throw ()
- *
- * @relates openvrml::vec3f
+ * @relatesalso vec3f
  *
  * @brief Add two vectors.
  *
@@ -719,6 +893,11 @@ const vec3f operator*(const mat4f & mat, const vec3f & vec) throw ()
  *
  * @return the result vector.
  */
+const vec3f operator+(const vec3f & lhs, const vec3f & rhs) throw ()
+{
+    vec3f result(lhs);
+    return result += rhs;
+}
 
 /**
  * @fn vec3f & vec3f::operator-=(const vec3f & vec) throw ()
@@ -729,11 +908,16 @@ const vec3f operator*(const mat4f & mat, const vec3f & vec) throw ()
  *
  * @return a reference to the object.
  */
+vec3f & vec3f::operator-=(const vec3f & vec) throw ()
+{
+    this->vec[0] -= vec[0];
+    this->vec[1] -= vec[1];
+    this->vec[2] -= vec[2];
+    return *this;
+}
 
 /**
- * @fn const vec3f operator-(const vec3f & lhs, const vec3f & rhs) throw ()
- *
- * @relates openvrml::vec3f
+ * @relatesalso vec3f
  *
  * @brief Subtract two vectors.
  *
@@ -742,6 +926,11 @@ const vec3f operator*(const mat4f & mat, const vec3f & vec) throw ()
  *
  * @return the result vector.
  */
+const vec3f operator-(const vec3f & lhs, const vec3f & rhs) throw ()
+{
+    vec3f result(lhs);
+    return result -= rhs;
+}
 
 /**
  * @fn const vec3f vec3f::operator-() const throw ()
@@ -750,9 +939,13 @@ const vec3f operator*(const mat4f & mat, const vec3f & vec) throw ()
  *
  * @return the additive inverse of the vector.
  */
+const vec3f vec3f::operator-() const throw()
+{
+    return vec3f(-this->vec[0], -this->vec[1], -this->vec[2]);
+}
 
 /**
- * @fn const float & vec3f::operator[](const size_t index) const throw ()
+ * @fn const float & vec3f::operator[](size_t index) const throw ()
  *
  * @brief Index-based component access.
  *
@@ -764,19 +957,6 @@ const vec3f operator*(const mat4f & mat, const vec3f & vec) throw ()
  * @pre @p index is less than 3.
  */
 
-/**
- * @fn float & vec3f::operator[](const size_t index) throw ()
- *
- * @brief Index-based component access.
- *
- * @param index 0 corresponds to the x component; 1 corresponds to the y
- *              component; 2 corresponds to the z component.
- *
- * @return a reference to the component corresponding to @p index.
- *
- * @pre @p index is less than 3.
- */
-
 /**
  * @fn float vec3f::x() const throw ()
  *
@@ -806,6 +986,8 @@ const vec3f operator*(const mat4f & mat, const vec3f & vec) throw ()
  *
  * @brief Set the x component.
  *
+ * @pre @p value is a valid numeric value (i.e., not NaN).
+ *
  * @param value new x component value.
  */
 
@@ -814,6 +996,8 @@ const vec3f operator*(const mat4f & mat, const vec3f & vec) throw ()
  *
  * @brief Set the y component.
  *
+ * @pre @p value is a valid numeric value (i.e., not NaN).
+ *
  * @param value new y component value.
  */
 
@@ -822,26 +1006,34 @@ const vec3f operator*(const mat4f & mat, const vec3f & vec) throw ()
  *
  * @brief Set the z component.
  *
+ * @pre @p value is a valid numeric value (i.e., not NaN).
+ *
  * @param value new z component value.
  */
 
 /**
- * @fn float vec3f::dot(const vec3f & vec) const throw ()
- *
  * @brief Dot product.
  *
  * @param vec
  *
  * @return the dot product of the vector and @p vec.
  */
+float vec3f::dot(const vec3f & vec) const throw()
+{
+    return this->vec[0] * vec[0]
+        + this->vec[1] * vec[1]
+        + this->vec[2] * vec[2];
+}
 
 /**
- * @fn float vec3f::length() const throw ()
- *
  * @brief Geometric length.
  *
  * @return the length of the vector.
  */
+float vec3f::length() const throw ()
+{
+    return float(sqrt(this->dot(*this)));
+}
 
 /**
  * @brief Normalize.
@@ -858,14 +1050,15 @@ const vec3f vec3f::normalize() const throw ()
 }
 
 /**
- * @relates vec3f
+ * @relatesalso vec3f
  *
  * @brief Compare for equality.
  *
  * @param lhs   left-hand operand.
  * @param rhs   right-hand operand.
  *
- * @return @c true if @p lhs and @p rhs have the same value; @c false otherwise.
+ * @return @c true if @p lhs and @p rhs have the same value; @c false
+ *         otherwise.
  */
 bool operator==(const vec3f & lhs, const vec3f & rhs) throw ()
 {
@@ -874,9 +1067,7 @@ bool operator==(const vec3f & lhs, const vec3f & rhs) throw ()
 }
 
 /**
- * @fn bool operator!=(const vec3f & lhs, const vec3f & rhs) throw ()
- *
- * @relates openvrml::vec3f
+ * @relatesalso vec3f
  *
  * @brief Compare for inequality.
  *
@@ -886,9 +1077,13 @@ bool operator==(const vec3f & lhs, const vec3f & rhs) throw ()
  * @return @c true if @p lhs and @p rhs dot not have the same value; @c false
  *      otherwise.
  */
+bool operator!=(const vec3f & lhs, const vec3f & rhs) throw ()
+{
+    return !(lhs == rhs);
+}
 
 /**
- * @relates vec3f
+ * @relatesalso vec3f
  *
  * @brief Stream output.
  *
@@ -927,10 +1122,15 @@ std::ostream & operator<<(std::ostream & out, const vec3f & v)
  */
 
 /**
- * @fn rotation::rotation() throw ()
- *
  * @brief Construct.
  */
+rotation::rotation() throw ()
+{
+    this->rot[0] = 0.0;
+    this->rot[1] = 0.0;
+    this->rot[2] = 1.0;
+    this->rot[3] = 0.0;
+}
 
 /**
  * @brief Construct from an array.
@@ -1052,9 +1252,7 @@ rotation & rotation::operator*=(const rotation & rot) throw ()
 }
 
 /**
- * @fn const rotation operator*(const rotation & lhs, const rotation & rhs) throw ()
- *
- * @relates openvrml::rotation
+ * @relatesalso rotation
  *
  * @brief Multiply rotations.
  *
@@ -1063,6 +1261,12 @@ rotation & rotation::operator*=(const rotation & rot) throw ()
  *
  * @return the product of @p lhs and @p rhs.
  */
+const rotation operator*(const rotation & lhs, const rotation & rhs) throw ()
+{
+    rotation result(lhs);
+    return result *= rhs;
+}
+
 
 /**
  * @fn const float & rotation::operator[](const size_t index) const throw ()
@@ -1202,12 +1406,16 @@ void rotation::axis(const vec3f & axis) throw ()
  */
 
 /**
- * @fn const rotation rotation::inverse() const throw ()
- *
  * @brief Inverse rotation.
  *
  * @return the inverse rotation.
  */
+const rotation rotation::inverse() const throw ()
+{
+    rotation result(*this);
+    result.rot[3] = -this->rot[3];
+    return result;
+}
 
 /**
  * @brief Spherical Linear intERPolation.
@@ -1270,7 +1478,7 @@ const rotation rotation::slerp(const rotation & dest_rot, const float t) const
 }
 
 /**
- * @relates rotation
+ * @relatesalso rotation
  *
  * @brief Compare for equality.
  *
@@ -1286,9 +1494,7 @@ bool operator==(const rotation & lhs, const rotation & rhs) throw ()
 }
 
 /**
- * @fn bool operator!=(const rotation & lhs, const rotation & rhs) throw ()
- *
- * @relates openvrml::rotation
+ * @relatesalso rotation
  *
  * @brief Compare for inequality.
  *
@@ -1297,9 +1503,13 @@ bool operator==(const rotation & lhs, const rotation & rhs) throw ()
  *
  * @return @c true if @p lhs is not equal to @p rhs; @c false otherwise.
  */
+bool operator!=(const rotation & lhs, const rotation & rhs) throw ()
+{
+    return !(lhs == rhs);
+}
 
 /**
- * @relates rotation
+ * @relatesalso rotation
  *
  * @brief Stream output.
  *
@@ -1615,9 +1825,7 @@ mat4f & mat4f::operator*=(const float scalar) throw ()
 }
 
 /**
- * @fn const mat4f operator*(const mat4f & mat, const float scalar) throw ()
- *
- * @relates openvrml::mat4f
+ * @relatesalso mat4f
  *
  * @brief Multiply a matrix by a scalar value.
  *
@@ -1626,11 +1834,14 @@ mat4f & mat4f::operator*=(const float scalar) throw ()
  *
  * @return the result matrix.
  */
+const mat4f operator*(const mat4f & mat, const float scalar) throw ()
+{
+    mat4f result(mat);
+    return result *= scalar;
+}
 
 /**
- * @fn const mat4f operator*(const float scalar, const mat4f & mat) throw ()
- *
- * @relates openvrml::mat4f
+ * @relatesalso mat4f
  *
  * @brief Multiply a scalar value by matrix.
  *
@@ -1639,6 +1850,11 @@ mat4f & mat4f::operator*=(const float scalar) throw ()
  *
  * @return the result matrix.
  */
+const mat4f operator*(const float scalar, const mat4f & mat) throw ()
+{
+    mat4f result(mat);
+    return result *= scalar;
+}
 
 /**
  * @brief Multiply by another matrix.
@@ -1682,9 +1898,7 @@ mat4f & mat4f::operator*=(const mat4f & mat) throw ()
 }
 
 /**
- * @fn const mat4f operator*(const mat4f & lhs, const mat4f & rhs) throw ()
- *
- * @relates openvrml::mat4f
+ * @relatesalso mat4f
  *
  * @brief Multiply two matrices.
  *
@@ -1693,6 +1907,12 @@ mat4f & mat4f::operator*=(const mat4f & mat) throw ()
  *
  * @return the result matrix.
  */
+const mat4f operator*(const mat4f & lhs, const mat4f & rhs) throw ()
+{
+    mat4f result(lhs);
+    return result *= rhs;
+}
+
 
 namespace {
 
@@ -1867,9 +2087,9 @@ void mat4f::transformation(vec3f & t,
     // Compute XY shear factor and make 2nd row orthogonal to 1st.
     //
     float shear_xy = row_0.dot(row_1);
-    row_1[0] = row_1[0] - row_0[0] * shear_xy;
-    row_1[1] = row_1[1] - row_0[1] * shear_xy;
-    row_1[2] = row_1[2] - row_0[2] * shear_xy;
+    row_1.x(row_1[0] - row_0[0] * shear_xy);
+    row_1.y(row_1[1] - row_0[1] * shear_xy);
+    row_1.z(row_1[2] - row_0[2] * shear_xy);
 
     //
     // Compute Y scale factor and normalize second row.
@@ -1882,13 +2102,13 @@ void mat4f::transformation(vec3f & t,
     // Compute XZ and YZ shears, orthogonalize third row.
     //
     float shear_xz = row_0.dot(row_2);
-    row_2[0] = row_2[0] - row_0[0] * shear_xz;
-    row_2[1] = row_2[1] - row_0[1] * shear_xz;
-    row_2[2] = row_2[2] - row_0[2] * shear_xz;
+    row_2.x(row_2[0] - row_0[0] * shear_xz);
+    row_2.y(row_2[1] - row_0[1] * shear_xz);
+    row_2.z(row_2[2] - row_0[2] * shear_xz);
     float shear_yz = row_1.dot(row_2);
-    row_2[0] = row_2[0] - row_1[0] * shear_yz;
-    row_2[1] = row_2[1] - row_1[1] * shear_yz;
-    row_2[2] = row_2[2] - row_1[2] * shear_yz;
+    row_2.x(row_2[0] - row_1[0] * shear_yz);
+    row_2.y(row_2[1] - row_1[1] * shear_yz);
+    row_2.z(row_2[2] - row_1[2] * shear_yz);
 
     //
     // Compute Z scale factor and normalize third row.
@@ -2052,7 +2272,7 @@ const mat4f mat4f::transpose() const throw ()
 }
 
 /**
- * @brief Return determinant of entire matrix.
+ * @brief Determinant.
  *
  * @return the determinant.
  */
@@ -2067,17 +2287,25 @@ float mat4f::det() const throw ()
 /**
  * @fn float (&mat4f::operator[](size_t index) throw ())[4]
  *
- * @brief To make it usual matrix (non-const)
+ * @brief Row access.
+ *
+ * @param index row index.
+ *
+ * @return row @p index.
  */
 
 /**
  * @fn float (&mat4f::operator[](size_t index) const throw ())[4]
  *
- * @brief To make it usual matrix (const)
+ * @brief Row access.
+ *
+ * @param index row index.
+ *
+ * @return row @p index.
  */
 
 /**
- * @relates mat4f
+ * @relatesalso mat4f
  *
  * @brief Stream output.
  *
@@ -2104,7 +2332,7 @@ std::ostream & operator<<(std::ostream & out, const mat4f & mat)
 
 
 /**
- * @relates mat4f
+ * @relatesalso mat4f
  *
  * @brief Equality comparison operator.
  *
@@ -2112,23 +2340,30 @@ std::ostream & operator<<(std::ostream & out, const mat4f & mat)
  *
  * @param lhs   a matrix.
  * @param rhs   a matrix.
+ *
+ * @return @c true if @p lhs and @p rhs are equivalent; @c false otherwise.
  */
 bool operator==(const mat4f & lhs, const mat4f & rhs) throw ()
 {
     using openvrml_::fequal;
-    return std::equal(&lhs[0][0], &lhs[0][0] + 16, &rhs[0][0], fequal<float>());
+    return std::equal(&lhs[0][0], &lhs[0][0] + 16, &rhs[0][0],
+                      fequal<float>());
 }
 
 /**
- * @fn bool operator!=(const mat4f & lhs, const mat4f & rhs) throw ()
- *
- * @relates openvrml::mat4f
+ * @relatesalso mat4f
  *
  * @brief Inequality comparison operator.
  *
  * @param lhs   a matrix.
  * @param rhs   a matrix.
+ *
+ * @return @c true if @p lhs and @p rhs are not equivalent; @c false otherwise.
  */
+bool operator!=(const mat4f & lhs, const mat4f & rhs) throw ()
+{
+    return !(lhs == rhs);
+}
 
 
 /**
@@ -2276,9 +2511,7 @@ quatf & quatf::operator*=(const quatf & quat) throw ()
 }
 
 /**
- * @fn const quatf operator*(const quatf & lhs, const quatf & rhs) throw ()
- *
- * @relates openvrml::quatf
+ * @relatesalso quatf
  *
  * @brief Multiply two quaternions.
  *
@@ -2287,6 +2520,11 @@ quatf & quatf::operator*=(const quatf & quat) throw ()
  *
  * @return the product of @p lhs and @p rhs.
  */
+const quatf operator*(const quatf & lhs, const quatf & rhs) throw ()
+{
+    quatf result(lhs);
+    return result *= rhs;
+}
 
 /**
  * @brief Multiply by a scalar.
@@ -2305,9 +2543,7 @@ quatf & quatf::operator*=(const float scalar) throw ()
 }
 
 /**
- * @fn const quatf operator*(const quatf & quat, float scalar) throw ()
- *
- * @relates openvrml::quatf
+ * @relatesalso quatf
  *
  * @brief Multiply a quaternion by a scalar.
  *
@@ -2316,11 +2552,14 @@ quatf & quatf::operator*=(const float scalar) throw ()
  *
  * @return the product of @p quat and @p scalar.
  */
+const quatf operator*(const quatf & quat, const float scalar) throw ()
+{
+    quatf result(quat);
+    return result *= scalar;
+}
 
 /**
- * @fn const quatf operator*(float scalar, const quatf & quat) throw ()
- *
- * @relates openvrml::quatf
+ * @relatesalso quatf
  *
  * @brief Multiply a scalar by a quaternion.
  *
@@ -2329,6 +2568,11 @@ quatf & quatf::operator*=(const float scalar) throw ()
  *
  * @return the product of @p scalar and @p quat.
  */
+const quatf operator*(const float scalar, const quatf & quat) throw ()
+{
+    quatf result(quat);
+    return result *= scalar;
+}
 
 /**
  * @brief Divide by a scalar.
@@ -2339,6 +2583,7 @@ quatf & quatf::operator*=(const float scalar) throw ()
  */
 quatf & quatf::operator/=(const float scalar) throw ()
 {
+    assert(scalar != 0.0);
     this->quat[0] /= scalar;
     this->quat[1] /= scalar;
     this->quat[2] /= scalar;
@@ -2347,9 +2592,7 @@ quatf & quatf::operator/=(const float scalar) throw ()
 }
 
 /**
- * @fn const quatf operator/(const quatf & quat, float scalar) throw ()
- *
- * @relates openvrml::quatf
+ * @relatesalso quatf
  *
  * @brief Divide a quaternion by a scalar.
  *
@@ -2358,6 +2601,11 @@ quatf & quatf::operator/=(const float scalar) throw ()
  *
  * @return the result of dividing @p quat by @p scalar.
  */
+const quatf operator/(const quatf & quat, const float scalar) throw ()
+{
+    quatf result(quat);
+    return result /= scalar;
+}
 
 /**
  * @brief Add a quaternion.
@@ -2376,9 +2624,7 @@ quatf & quatf::operator+=(const quatf & quat) throw ()
 }
 
 /**
- * @fn const quatf operator+(const quatf & lhs, const quatf & rhs) throw ()
- *
- * @relates openvrml::quatf
+ * @relatesalso quatf
  *
  * @brief Add two quaternions.
  *
@@ -2387,6 +2633,11 @@ quatf & quatf::operator+=(const quatf & quat) throw ()
  *
  * @return the sum of @p lhs and @p rhs.
  */
+const quatf operator+(const quatf & lhs, const quatf & rhs) throw ()
+{
+    quatf result(lhs);
+    return result += rhs;
+}
 
 /**
  * @brief Subtract a quaternion.
@@ -2405,9 +2656,7 @@ quatf & quatf::operator-=(const quatf & quat) throw ()
 }
 
 /**
- * @fn const quatf operator-(const quatf & lhs, const quatf & rhs) throw ()
- *
- * @relates openvrml::quatf
+ * @relatesalso quatf
  *
  * @brief Take the difference between two quaternions.
  *
@@ -2416,6 +2665,11 @@ quatf & quatf::operator-=(const quatf & quat) throw ()
  *
  * @return the difference between @p lhs and @p rhs.
  */
+const quatf operator-(const quatf & lhs, const quatf & rhs) throw ()
+{
+    quatf result(lhs);
+    return result -= rhs;
+}
 
 /**
  * @fn float quatf::operator[](size_t index) const throw ()
@@ -2564,7 +2818,7 @@ const quatf quatf::normalize() const throw ()
 }
 
 /**
- * @relates quatf
+ * @relatesalso quatf
  *
  * @brief Stream output.
  *
@@ -2580,7 +2834,7 @@ std::ostream & operator<<(std::ostream & out, const quatf & quat)
 }
 
 /**
- * @relates quatf
+ * @relatesalso quatf
  *
  * @brief Compare for equality.
  *
@@ -2596,9 +2850,7 @@ bool operator==(const quatf & lhs, const quatf & rhs) throw ()
 }
 
 /**
- * @fn bool operator!=(const quatf & lhs, const quatf & rhs) throw ()
- *
- * @relates openvrml::quatf
+ * @relatesalso quatf
  *
  * @brief Compare for inequality.
  *
@@ -2607,5 +2859,343 @@ bool operator==(const quatf & lhs, const quatf & rhs) throw ()
  *
  * @return @c true if @p lhs and @p rhs are not equal; @c false otherwise.
  */
+bool operator!=(const quatf & lhs, const quatf & rhs) throw ()
+{
+    return !(lhs == rhs);
+}
+
+
+/**
+ * @class image
+ *
+ * @ingroup basetypes
+ *
+ * @brief Pixmap data.
+ *
+ * The first pixel in the @link image::array array@endlink is the lower left
+ * pixel and the last is the upper right pixel.  Pixel values are limited
+ * to 256 levels of intensity.  The elements of
+ * @link image::array array@endlink are bytes, with one byte per pixel
+ * component.  Thus the number of elements in the array is
+ * @p x * &p y * @p comp.
+ *
+ * A one-component image specifies one-byte greyscale values.  A two-component
+ * image specifies the intensity in the first byte and the alpha opacity in the
+ * second byte.  A three-component image specifies the red component in the
+ * first byte, followed by the green and blue components.  Four-component
+ * images specify the alpha opacity byte after red/green/blue.
+ */
+
+/**
+ * @var size_t image::x_
+ *
+ * @brief Pixels in the <var>x</var>-dimension.
+ */
+
+/**
+ * @var size_t image::y_
+ *
+ * @brief Pixels in the <var>y</var>-dimension.
+ */
+
+/**
+ * @var size_t image::comp_
+ *
+ * @brief Number of components.
+ */
+
+/**
+ * @var std::vector<int32> image::array_
+ *
+ * @brief Pixel data.
+ */
+
+/**
+ * @brief Construct.
+ */
+image::image() throw ()
+{}
+
+/**
+ * @brief Construct.
+ *
+ * @param x     pixels in the <var>x</var>-direction.
+ * @param y     pixels in the <var>y</var>-direction.
+ * @param comp  number of components.
+ */
+image::image(const size_t x,
+             const size_t y,
+             const size_t comp)
+    throw (std::bad_alloc):
+    x_(x),
+    y_(y),
+    comp_(comp),
+    array_(x * y * comp)
+{}
+
+/**
+ * @brief Construct.
+ *
+ * @pre @p array.size() <= @p x * @p y & @p comp.
+ *
+ * @param x     pixels in the <var>x</var>-direction.
+ * @param y     pixels in the <var>y</var>-direction.
+ * @param comp  number of components.
+ * @param array pixel data.
+ */
+image::image(const size_t x,
+             const size_t y,
+             const size_t comp,
+             const std::vector<unsigned char> & array)
+    throw (std::bad_alloc):
+    x_(x),
+    y_(y),
+    comp_(comp),
+    array_(array)
+{
+    assert(array.size() <= x * y * comp);
+}
+
+/**
+ * @fn template <typename InputIterator> image::image(size_t x, size_t y, size_t comp, InputIterator array_begin, InputIterator array_end) throw (std::bad_alloc)
+ *
+ * @brief Construct.
+ *
+ * @pre std::distance(@p array_begin, @p array_end) <= @p x * @p y * @p comp.
+ *
+ * @param x             pixels in the <var>x</var>-direction.
+ * @param y             pixels in the <var>y</var>-direction.
+ * @param comp          number of components.
+ * @param array_begin   pixel data begin iterator.
+ * @param array_end     pixel data end iterator.
+ *
+ * @exception std::bad_alloc    if memory allocation fails.
+ */
+
+/**
+ * @fn size_t image::x() const throw ()
+ *
+ * @brief Pixels in the <var>x</var>-dimension.
+ *
+ * @return the number of pixels in the <var>x</var>-dimension.
+ */
+
+/**
+ * @brief Set the pixels in the <var>x</var>-dimension.
+ *
+ * @param value new value for the <var>x</var>-dimension.
+ *
+ * @exception std::bad_alloc    if memory allocation fails.
+ */
+void image::x(const size_t value) throw (std::bad_alloc)
+{
+    //
+    // Throws std::bad_alloc.
+    //
+    this->array_.resize(value * this->y_ * this->comp_);
+    this->x_ = value;
+}
+
+/**
+ * @fn size_t image::y() const throw ()
+ *
+ * @brief Pixels in the <var>y</var>-dimension.
+ *
+ * @return the number of pixels in the <var>y</var>-dimension.
+ */
+
+/**
+ * @brief Set the pixels in the <var>y</var>-dimension.
+ *
+ * @param value new value for the <var>x</var>-dimension.
+ *
+ * @exception std::bad_alloc    if memory allocation fails.
+ */
+void image::y(const size_t value) throw (std::bad_alloc)
+{
+    //
+    // Throws std::bad_alloc.
+    //
+    this->array_.resize(this->x_ * value * this->comp_);
+    this->y_ = value;
+}
+
+/**
+ * @brief Resize the <var>x</var>- and <var>y</var>-dimensions.
+ *
+ * @param x <var>x</var>-dimension.
+ * @param y <var>y</var>-dimension.
+ *
+ * @exception std::bad_alloc    if memory allocation fails.
+ */
+void image::resize(const size_t x, const size_t y) throw (std::bad_alloc)
+{
+    this->array_.resize(x * y * this->comp_); // Throws std::bad_alloc.
+    this->x_ = x;
+    this->y_ = y;
+}
+
+/**
+ * @fn size_t image::comp() const throw ()
+ *
+ * @brief Number of components.
+ *
+ * @return the number of components.
+ */
+
+/**
+ * @brief Set the number of components.
+ *
+ * @pre @p value <= 4
+ *
+ * @param value number of components.
+ */
+ void image::comp(const size_t value) throw (std::bad_alloc)
+{
+    assert(value <= 4);
+    this->array_.resize(this->x_ * this->y_ * value);
+    this->comp_ = value;
+}
+
+
+/**
+ * @fn const std::vector<unsigned char> & image::array() const throw ()
+ *
+ * @brief Pixel value array.
+ *
+ * @return the array of pixel values.
+ */
+
+/**
+ * @fn void image::array(const std::vector<unsigned char> & value) throw ()
+ *
+ * @brief Set the pixel value array.
+ *
+ * @pre @p value.size() <= (x() * y() * comp())
+ *
+ * @param value pixel value array.
+ */
+
+/**
+ * @fn template <typename InputIterator> void image::array(InputIterator begin, InputIterator end) throw ()
+ *
+ * @brief Set the pixel value array.
+ *
+ * @pre std::distance(@p begin, @p end) <= (x() * y() * comp())
+ *
+ * @param begin input iterator to the beginning of a sequence.
+ * @param end   input iterator to the end of the sequence (one past the last
+ *              element).
+ */
+
+/**
+ * @fn int32 image::pixel(size_t index) const throw ()
+ *
+ * @brief Pixel value.
+ *
+ * @pre @p index < x() * y()
+ *
+ * @param index pixel value index.
+ *
+ * @return the pixel value at @p index.
+ */
+
+/**
+ * @fn void image::pixel(size_t index, int32 value) throw ()
+ *
+ * @brief Set a pixel value.
+ *
+ * @pre @p index < x() * y()
+ *
+ * @param index pixel value index.
+ * @param value pixel value.
+ */
+
+/**
+ * @fn int32 image::pixel(size_t x, size_t y) const throw ()
+ *
+ * @brief Pixel value.
+ *
+ * @pre @p x < x(), @p y < y()
+ *
+ * @param x pixel value <var>x</var>- (column) index.
+ * @param y pixel value <var>y</var>- (row) index.
+ *
+ * @return the pixel value at (@p x, @p y).
+ */
+
+/**
+ * @fn void image::pixel(size_t x, size_t y, int32 value) throw ()
+ *
+ * @brief Set a pixel value.
+ *
+ * @pre @p x < x(), @p y < y()
+ *
+ * @param x     pixel value <var>x</var>- (column) index.
+ * @param y     pixel value <var>y</var>- (row) index.
+ * @param value pixel value.
+ */
+
+/**
+ * @relatesalso image
+ *
+ * @brief Compare for equality.
+ *
+ * @param lhs   left-hand operand.
+ * @param rhs   right-hand operand.
+ *
+ * @return @c true if @p lhs and @p rhs are equal; @c false otherwise.
+ */
+bool operator==(const image & lhs, const image & rhs) throw ()
+{
+    return lhs.x() == rhs.x()
+        && lhs.y() == rhs.y()
+        && lhs.comp() == rhs.comp()
+        && lhs.array() == rhs.array();
+}
+
+/**
+ * @relatesalso image
+ *
+ * @brief Compare for inequality.
+ *
+ * @param lhs   left-hand operand.
+ * @param rhs   right-hand operand.
+ *
+ * @return @c true if @p lhs and @p rhs are not equal; @c false otherwise.
+ */
+bool operator!=(const image & lhs, const image & rhs) throw ()
+{
+    return !(lhs == rhs);
+}
+
+/**
+ * @relatesalso image
+ *
+ * @brief Stream output.
+ *
+ * @param out   output stream.
+ * @param img   image.
+ */
+std::ostream & operator<<(std::ostream & out, const image & img)
+{
+    using std::vector;
+    using std::ios_base;
+    using std::dec;
+    using std::hex;
+
+    const ios_base::fmtflags save_flags = out.flags();
+    out << dec << img.x() << ' ' << img.y() << ' ' << img.comp() << hex;
+    for (size_t pixel_index = 0;
+         pixel_index < img.x() * img.y();
+         ++pixel_index) {
+        out << ' ' << "0x";
+        for (size_t component = 0; component < img.comp(); ++component) {
+            out << int(img.array()[pixel_index * img.comp() + component]);
+        }
+    }
+    out.flags(save_flags);
+    return out;
+}
 
 }
