@@ -2,8 +2,8 @@
 //
 // OpenVRML
 //
-// Copyright (C) 2001  S. K. Bose
-// Copyright (C) 2003  Braden McDaniel
+// Copyright 2001  S. K. Bose
+// Copyright 2003, 2004  Braden McDaniel
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -28,6 +28,7 @@
 #   include <cstddef>
 #   include <algorithm>
 #   include <iosfwd>
+#   include <vector>
 #   include <sys/types.h>
 #   include <openvrml/common.h>
 
@@ -61,28 +62,9 @@ namespace openvrml {
         void hsv(float h, float s, float v) throw ();
     };
 
-    inline color::color() throw ()
-    {}
-
-    inline color::color(const float (&rgb)[3]) throw ()
-    {
-        assert(rgb[0] >= 0.0 && rgb[0] <= 1.0);
-        this->rgb[0] = rgb[0];
-        assert(rgb[1] >= 0.0 && rgb[1] <= 1.0);
-        this->rgb[1] = rgb[1];
-        assert(rgb[2] >= 0.0 && rgb[2] <= 1.0);
-        this->rgb[2] = rgb[2];
-    }
-
-    inline color::color(float r, float g, float b) throw ()
-    {
-        assert(r >= 0.0 && r <= 1.0);
-        this->rgb[0] = r;
-        assert(g >= 0.0 && g <= 1.0);
-        this->rgb[1] = g;
-        assert(b >= 0.0 && b <= 1.0);
-        this->rgb[2] = b;
-    }
+    bool operator==(const color & lhs, const color & rhs) throw ();
+    bool operator!=(const color & lhs, const color & rhs) throw ();
+    std::ostream & operator<<(std::ostream & out, const color & c);
 
     inline const float & color::operator[](const size_t index) const throw ()
     {
@@ -120,16 +102,6 @@ namespace openvrml {
         this->rgb[2] = value;
     }
 
-    bool operator==(const color & lhs, const color & rhs)
-        throw ();
-
-    inline bool operator!=(const color & lhs, const color & rhs) throw ()
-    {
-        return !(lhs == rhs);
-    }
-
-    std::ostream & operator<<(std::ostream & out, const color & c);
-
 
     class vec2f {
         float vec[2];
@@ -147,7 +119,6 @@ namespace openvrml {
         const vec2f operator-() const throw ();
 
         const float & operator[](size_t index) const throw ();
-        float & operator[](size_t index) throw ();
 
         float x() const throw ();
         float y() const throw ();
@@ -160,80 +131,14 @@ namespace openvrml {
         const vec2f normalize() const throw ();
     };
 
-    inline vec2f::vec2f() throw ()
-    {
-        this->vec[0] = 0.0;
-        this->vec[1] = 0.0;
-    }
-
-    inline vec2f::vec2f(const float (&vec)[2]) throw ()
-    {
-        this->vec[0] = vec[0];
-        this->vec[1] = vec[1];
-    }
-
-    inline vec2f::vec2f(const float x, const float y) throw ()
-    {
-        this->vec[0] = x;
-        this->vec[1] = y;
-    }
-
-    inline vec2f & vec2f::operator*=(const float scalar) throw ()
-    {
-        this->vec[0] *= scalar;
-        this->vec[1] *= scalar;
-        return *this;
-    }
-
-    inline const vec2f operator*(const vec2f & lhs, const float rhs) throw ()
-    {
-        vec2f result(lhs);
-        return result *= rhs;
-    }
-
-    inline vec2f & vec2f::operator/=(const float scalar) throw ()
-    {
-        this->vec[0] /= scalar;
-        this->vec[1] /= scalar;
-        return *this;
-    }
-
-    inline const vec2f operator/(const vec2f & lhs, const float rhs) throw ()
-    {
-        vec2f result(lhs);
-        return result /= rhs;
-    }
-
-    inline vec2f & vec2f::operator+=(const vec2f & vec) throw ()
-    {
-        this->vec[0] += vec[0];
-        this->vec[1] += vec[1];
-        return *this;
-    }
-
-    inline const vec2f operator+(const vec2f & lhs, const vec2f & rhs) throw ()
-    {
-        vec2f result(lhs);
-        return result += rhs;
-    }
-
-    inline vec2f & vec2f::operator-=(const vec2f & vec) throw ()
-    {
-        this->vec[0] -= vec[0];
-        this->vec[1] -= vec[1];
-        return *this;
-    }
-
-    inline const vec2f operator-(const vec2f & lhs, const vec2f & rhs) throw ()
-    {
-        vec2f result(lhs);
-        return result -= rhs;
-    }
-
-    inline const vec2f vec2f::operator-() const throw ()
-    {
-        return vec2f(-this->vec[0], -this->vec[1]);
-    }
+    const vec2f operator*(const vec2f & lhs, float rhs) throw ();
+    const vec2f operator*(float lhs, const vec2f & rhs) throw ();
+    const vec2f operator/(const vec2f & lhs, float rhs) throw ();
+    const vec2f operator+(const vec2f & lhs, const vec2f & rhs) throw ();
+    const vec2f operator-(const vec2f & lhs, const vec2f & rhs) throw ();
+    bool operator==(const vec2f & lhs, const vec2f & rhs) throw ();
+    bool operator!=(const vec2f & lhs, const vec2f & rhs) throw ();
+    std::ostream & operator<<(std::ostream & out, const vec2f & v);
 
     inline const float & vec2f::operator[](const size_t index) const throw ()
     {
@@ -241,12 +146,6 @@ namespace openvrml {
         return this->vec[index];
     }
 
-    inline float & vec2f::operator[](const size_t index) throw ()
-    {
-        assert(index < 2);
-        return this->vec[index];
-    }
-
     inline float vec2f::x() const throw ()
     {
         return this->vec[0];
@@ -259,35 +158,24 @@ namespace openvrml {
 
     inline void vec2f::x(const float value) throw ()
     {
+        //
+        // Ensure value is not NaN.
+        //
+        assert(value == value);
+
         this->vec[0] = value;
     }
 
     inline void vec2f::y(const float value) throw ()
     {
+        //
+        // Ensure value is not NaN.
+        //
+        assert(value == value);
+
         this->vec[1] = value;
     }
 
-    inline float vec2f::dot(const vec2f & vec) const throw ()
-    {
-        return this->vec[0] * vec[0]
-             + this->vec[1] * vec[1];
-    }
-
-    inline float vec2f::length() const throw ()
-    {
-        return float(sqrt(this->dot(*this)));
-    }
-
-    bool operator==(const vec2f & lhs, const vec2f & rhs)
-        throw ();
-
-    inline bool operator!=(const vec2f & lhs, const vec2f & rhs) throw ()
-    {
-        return !(lhs == rhs);
-    }
-
-    std::ostream & operator<<(std::ostream & out, const vec2f & v);
-
 
     class mat4f;
 
@@ -309,7 +197,6 @@ namespace openvrml {
         const vec3f operator-() const throw ();
 
         const float & operator[](size_t index) const throw ();
-        float & operator[](size_t index) throw ();
 
         float x() const throw ();
         float y() const throw ();
@@ -324,112 +211,17 @@ namespace openvrml {
         const vec3f normalize() const throw ();
     };
 
-    inline vec3f::vec3f() throw ()
-    {
-        this->vec[0] = 0.0;
-        this->vec[1] = 0.0;
-        this->vec[2] = 0.0;
-    }
-
-    inline vec3f::vec3f(const float (&vec)[3]) throw ()
-    {
-        this->vec[0] = vec[0];
-        this->vec[1] = vec[1];
-        this->vec[2] = vec[2];
-    }
-
-    inline vec3f::vec3f(const float x, const float y, const float z) throw ()
-    {
-        this->vec[0] = x;
-        this->vec[1] = y;
-        this->vec[2] = z;
-    }
-
-    inline vec3f & vec3f::operator*=(const vec3f & vec) throw ()
-    {
-        vec3f temp;
-        temp.x(this->y() * vec.z() - this->z() * vec.y());
-        temp.y(this->z() * vec.x() - this->x() * vec.z());
-        temp.z(this->x() * vec.y() - this->y() * vec.x());
-        *this = temp;
-        return *this;
-    }
-
-    inline const vec3f operator*(const vec3f & lhs, const vec3f & rhs) throw ()
-    {
-        vec3f result(lhs);
-        return result *= rhs;
-    }
-
-    inline vec3f & vec3f::operator*=(const float scalar) throw ()
-    {
-        this->vec[0] *= scalar;
-        this->vec[1] *= scalar;
-        this->vec[2] *= scalar;
-        return *this;
-    }
-
-    inline const vec3f operator*(const vec3f & lhs, const float rhs) throw ()
-    {
-        vec3f result(lhs);
-        return result *= rhs;
-    }
-
-    inline const vec3f operator*(const vec3f & vec, const mat4f & mat) throw ()
-    {
-        vec3f result(vec);
-        return result *= mat;
-    }
-
-    const vec3f operator*(const mat4f & mat, const vec3f & vec)
-        throw ();
-
-    inline vec3f & vec3f::operator/=(const float scalar) throw ()
-    {
-        this->vec[0] /= scalar;
-        this->vec[1] /= scalar;
-        this->vec[2] /= scalar;
-        return *this;
-    }
-
-    inline const vec3f operator/(const vec3f & lhs, const float rhs) throw ()
-    {
-        vec3f result(lhs);
-        return result /= rhs;
-    }
-
-    inline vec3f & vec3f::operator+=(const vec3f & vec) throw ()
-    {
-        this->vec[0] += vec[0];
-        this->vec[1] += vec[1];
-        this->vec[2] += vec[2];
-        return *this;
-    }
-
-    inline const vec3f operator+(const vec3f & lhs, const vec3f & rhs) throw ()
-    {
-        vec3f result(lhs);
-        return result += rhs;
-    }
-
-    inline vec3f & vec3f::operator-=(const vec3f & vec) throw ()
-    {
-        this->vec[0] -= vec[0];
-        this->vec[1] -= vec[1];
-        this->vec[2] -= vec[2];
-        return *this;
-    }
-
-    inline const vec3f operator-(const vec3f & lhs, const vec3f & rhs) throw ()
-    {
-        vec3f result(lhs);
-        return result -= rhs;
-    }
-
-    inline const vec3f vec3f::operator-() const throw()
-    {
-        return vec3f(-this->vec[0], -this->vec[1], -this->vec[2]);
-    }
+    const vec3f operator*(const vec3f & lhs, const vec3f & rhs) throw ();
+    const vec3f operator*(const vec3f & lhs, const float rhs) throw ();
+    const vec3f operator*(const float lhs, const vec3f & rhs) throw ();
+    const vec3f operator*(const vec3f & vec, const mat4f & mat) throw ();
+    const vec3f operator*(const mat4f & mat, const vec3f & vec) throw ();
+    const vec3f operator/(const vec3f & lhs, const float rhs) throw ();
+    const vec3f operator+(const vec3f & lhs, const vec3f & rhs) throw ();
+    const vec3f operator-(const vec3f & lhs, const vec3f & rhs) throw ();
+    bool operator==(const vec3f & lhs, const vec3f & rhs) throw ();
+    bool operator!=(const vec3f & lhs, const vec3f & rhs) throw ();
+    std::ostream & operator<<(std::ostream & out, const vec3f & v);
 
     inline const float & vec3f::operator[](const size_t index) const throw ()
     {
@@ -437,12 +229,6 @@ namespace openvrml {
         return this->vec[index];
     }
 
-    inline float & vec3f::operator[](const size_t index) throw ()
-    {
-        assert(index < 3);
-        return this->vec[index];
-    }
-
     inline float vec3f::x() const throw ()
     {
         return this->vec[0];
@@ -460,41 +246,34 @@ namespace openvrml {
 
     inline void vec3f::x(const float value) throw ()
     {
+        //
+        // Ensure value is not NaN.
+        //
+        assert(value == value);
+
         this->vec[0] = value;
     }
 
     inline void vec3f::y(const float value) throw ()
     {
+        //
+        // Ensure value is not NaN.
+        //
+        assert(value == value);
+
         this->vec[1] = value;
     }
 
     inline void vec3f::z(const float value) throw ()
     {
+        //
+        // Ensure value is not NaN.
+        //
+        assert(value == value);
+
         this->vec[2] = value;
     }
 
-    inline float vec3f::dot(const vec3f & vec) const throw()
-    {
-        return this->vec[0] * vec[0]
-             + this->vec[1] * vec[1]
-             + this->vec[2] * vec[2];
-    }
-
-    inline float vec3f::length() const throw ()
-    {
-        return float(sqrt(this->dot(*this)));
-    }
-
-    bool operator==(const vec3f & lhs, const vec3f & rhs)
-        throw ();
-
-    inline bool operator!=(const vec3f & lhs, const vec3f & rhs) throw ()
-    {
-        return !(lhs == rhs);
-    }
-
-    std::ostream & operator<<(std::ostream & out, const vec3f & v);
-
 
     class quatf;
 
@@ -530,20 +309,11 @@ namespace openvrml {
             throw ();
     };
 
-    inline const rotation operator*(const rotation & lhs, const rotation & rhs)
-        throw ()
-    {
-        rotation result(lhs);
-        return result *= rhs;
-    }
-
-    inline rotation::rotation() throw ()
-    {
-        this->rot[0] = 0.0;
-        this->rot[1] = 0.0;
-        this->rot[2] = 1.0;
-        this->rot[3] = 0.0;
-    }
+    const rotation operator*(const rotation & lhs, const rotation & rhs)
+        throw ();
+    bool operator==(const rotation & lhs, const rotation & rhs) throw ();
+    bool operator!=(const rotation & lhs, const rotation & rhs) throw ();
+    std::ostream & operator<<(std::ostream & out, const rotation & r);
 
     inline const float & rotation::operator[](const size_t index) const
         throw ()
@@ -582,23 +352,6 @@ namespace openvrml {
         this->rot[3] = value;
     }
 
-    inline const rotation rotation::inverse() const throw ()
-    {
-        rotation result(*this);
-        result.rot[3] = -this->rot[3];
-        return result;
-    }
-
-    bool operator==(const rotation & lhs, const rotation & rhs)
-        throw ();
-
-    inline bool operator!=(const rotation & lhs, const rotation & rhs) throw ()
-    {
-        return !(lhs == rhs);
-    }
-
-    std::ostream & operator<<(std::ostream & out, const rotation & r);
-
 
     class mat4f {
         float mat[4][4];
@@ -628,8 +381,8 @@ namespace openvrml {
         mat4f & operator*=(float scalar) throw ();
         mat4f & operator*=(const mat4f & mat) throw ();
 
-        float (&operator[](size_t index) throw ())[4];
-        const float (&operator[](size_t index) const throw ())[4];
+        float (&operator[](const size_t index) throw ())[4];
+        const float (&operator[](const size_t index) const throw ())[4];
 
         void transformation(vec3f & t, openvrml::rotation & r, vec3f & s) const
             throw ();
@@ -642,14 +395,8 @@ namespace openvrml {
         float det() const throw ();
     };
 
-    bool operator==(const mat4f & lhs, const mat4f & rhs)
-        throw ();
-
-    inline bool operator!=(const mat4f & lhs, const mat4f & rhs)
-        throw ()
-    {
-        return !(lhs == rhs);
-    }
+    bool operator==(const mat4f & lhs, const mat4f & rhs) throw ();
+    bool operator!=(const mat4f & lhs, const mat4f & rhs) throw ();
 
     inline float (&mat4f::operator[](size_t index) throw ())[4]
     {
@@ -663,26 +410,9 @@ namespace openvrml {
         return this->mat[index];
     }
 
-    inline const mat4f operator*(const mat4f & lhs, const mat4f & rhs) throw ()
-    {
-        mat4f result(lhs);
-        return result *= rhs;
-    }
-
-    inline const mat4f operator*(const mat4f & mat, const float scalar)
-        throw ()
-    {
-        mat4f result(mat);
-        return result *= scalar;
-    }
-
-    inline const mat4f operator*(const float scalar, const mat4f & mat)
-        throw ()
-    {
-        mat4f result(mat);
-        return result *= scalar;
-    }
-
+    const mat4f operator*(const mat4f & lhs, const mat4f & rhs) throw ();
+    const mat4f operator*(const mat4f & mat, float scalar) throw ();
+    const mat4f operator*(float scalar, const mat4f & mat) throw ();
     std::ostream & operator<<(std::ostream & out, const mat4f & mat);
 
 
@@ -723,52 +453,15 @@ namespace openvrml {
         const quatf normalize() const throw ();
     };
 
-    bool operator==(const quatf & lhs, const quatf & rhs)
-        throw ();
-
-    inline bool operator!=(const quatf & lhs, const quatf & rhs) throw ()
-    {
-        return !(lhs == rhs);
-    }
-
-    inline const quatf operator*(const quatf & lhs, const quatf & rhs) throw ()
-    {
-        quatf result(lhs);
-        return result *= rhs;
-    }
-
-    inline const quatf operator*(const quatf & quat, const float scalar)
-        throw ()
-    {
-        quatf result(quat);
-        return result *= scalar;
-    }
-
-    inline const quatf operator*(const float scalar, const quatf & quat)
-        throw ()
-    {
-        quatf result(quat);
-        return result *= scalar;
-    }
-
-    inline const quatf operator/(const quatf & quat, const float scalar)
-        throw ()
-    {
-        quatf result(quat);
-        return result /= scalar;
-    }
-
-    inline const quatf operator+(const quatf & lhs, const quatf & rhs) throw ()
-    {
-        quatf result(lhs);
-        return result += rhs;
-    }
-
-    inline const quatf operator-(const quatf & lhs, const quatf & rhs) throw ()
-    {
-        quatf result(lhs);
-        return result -= rhs;
-    }
+    const quatf operator*(const quatf & lhs, const quatf & rhs) throw ();
+    const quatf operator*(const quatf & quat, const float scalar) throw ();
+    const quatf operator*(const float scalar, const quatf & quat) throw ();
+    const quatf operator/(const quatf & quat, const float scalar) throw ();
+    const quatf operator+(const quatf & lhs, const quatf & rhs) throw ();
+    const quatf operator-(const quatf & lhs, const quatf & rhs) throw ();
+    bool operator==(const quatf & lhs, const quatf & rhs) throw ();
+    bool operator!=(const quatf & lhs, const quatf & rhs) throw ();
+    std::ostream & operator<<(std::ostream & out, const quatf & quat);
 
     inline const float & quatf::operator[](size_t index) const throw ()
     {
@@ -822,7 +515,141 @@ namespace openvrml {
         this->quat[3] = value;
     }
 
-    std::ostream & operator<<(std::ostream & out, const quatf & quat);
+
+    class image {
+        size_t x_;
+        size_t y_;
+        size_t comp_;
+        std::vector<unsigned char> array_;
+
+    public:
+        image() throw ();
+        image(size_t x, size_t y, size_t comp) throw (std::bad_alloc);
+        image(size_t x, size_t y, size_t comp,
+              const std::vector<unsigned char> & array) throw (std::bad_alloc);
+        template <typename InputIterator>
+        image(size_t x, size_t y, size_t comp,
+              InputIterator array_begin, InputIterator array_end)
+            throw (std::bad_alloc);
+
+        // Use compiler-defined operator= and copy constructor.
+
+        size_t x() const throw ();
+        void x(size_t value) throw (std::bad_alloc);
+
+        size_t y() const throw ();
+        void y(size_t value) throw (std::bad_alloc);
+
+        void resize(size_t x, size_t y) throw (std::bad_alloc);
+
+        size_t comp() const throw ();
+        void comp(size_t value) throw (std::bad_alloc);
+
+        const std::vector<unsigned char> & array() const throw ();
+        void array(const std::vector<unsigned char> & value) throw ();
+        template <typename InputIterator>
+        void array(InputIterator begin, InputIterator end) throw ();
+
+        int32 pixel(size_t index) const throw ();
+        void pixel(size_t index, int32 value) throw ();
+        int32 pixel(size_t x, size_t y) const throw ();
+        void pixel(size_t x, size_t y, int32 value) throw ();
+    };
+
+    bool operator==(const image & lhs, const image & rhs) throw ();
+    bool operator!=(const image & lhs, const image & rhs) throw ();
+
+    template <typename InputIterator>
+    image::image(const size_t x,
+                 const size_t y,
+                 const size_t comp,
+                 InputIterator array_begin,
+                 InputIterator array_end)
+        throw (std::bad_alloc):
+        x_(x),
+        y_(y),
+        comp_(comp),
+        array_(array_begin, array_end)
+    {
+        using std::iterator_traits;
+        using std::distance;
+        typedef typename iterator_traits<InputIterator>::difference_type
+            difference_type;
+        assert(distance(array_begin, array_end)
+               <= difference_type(x * y * comp));
+    }
+
+    inline size_t image::x() const throw ()
+    {
+        return this->x_;
+    }
+
+    inline size_t image::y() const throw ()
+    {
+        return this->y_;
+    }
+
+    inline size_t image::comp() const throw ()
+    {
+        return this->comp_;
+    }
+
+    inline const std::vector<unsigned char> & image::array() const throw ()
+    {
+        return this->array_;
+    }
+
+    inline void image::array(const std::vector<unsigned char> & value) throw ()
+    {
+        assert(value.size() <= (this->x_ * this->y_ * this->comp_));
+        std::copy(value.begin(), value.end(), this->array_.begin());
+    }
+
+    template <typename InputIterator>
+    void image::array(InputIterator begin, InputIterator end) throw ()
+    {
+        assert(std::distance(begin, end) <= this->x_ * this->y_ * this->comp_);
+        std::copy(begin, end, this->array_.begin());
+    }
+
+    inline int32 image::pixel(const size_t index) const throw ()
+    {
+        assert(index < this->x_ * this->y_);
+        int32 retval = 0x00000000;
+        for (size_t component = this->comp_, i = index * this->comp_;
+             component > 0;
+             --component, ++i) {
+            retval |= int32(this->array_[i]) << (8 * component);
+        }
+        return retval;
+    }
+
+    inline void image::pixel(const size_t index, const int32 value) throw ()
+    {
+        assert(index < this->x_ * this->y_);
+        for (size_t component = this->comp_, i = index * this->comp_;
+             component > 0;
+             --component, ++i) {
+            this->array_[i] =
+                static_cast<unsigned char>(
+                    (value >> (8 * (component - 1))) & 0x000000ff);
+        }
+    }
+
+    inline int32 image::pixel(const size_t x, const size_t y) const throw ()
+    {
+        assert((x * y) < this->array_.size());
+        return this->pixel(y * this->x_ + x);
+    }
+
+    inline void image::pixel(const size_t x, const size_t y, const int32 value)
+        throw ()
+    {
+        assert((x * y) < this->array_.size());
+        this->pixel(y * this->x_ + x, value);
+    }
+
+    std::ostream & operator<<(std::ostream & out, const image & img);
 
 } // namespace openvrml
 
