@@ -25,18 +25,20 @@
 #   include <cstddef>
 #   include <string>
 #   include <openvrml/common.h>
+#   include <boost/utility.hpp>
 
 namespace openvrml {
 
     class mfstring;
 
-    class system {
+    class system : boost::noncopyable {
     public:
+        virtual ~system() throw ();
         virtual bool load_url(const std::string & url,
                               const mfstring & parameters);
         virtual int connect_socket( const char *host, int port );
         virtual const char *http_host(const char *url, int *port);
-        virtual const char *http_fetch( const char *url );
+        virtual const std::string http_fetch( const char *url );
         virtual void remove_file( const char *fn );
     };
 
